Package: BOBaFIT
Type: Package
Title: Refitting diploid region profiles using a clustering procedure
Version: 1.16.0
Authors@R: c(person("Andrea", "Poletti", email="andrea.poletti5@gmail.com", 
      role=c("aut")),
      person("Gaia", "Mazzocchetti", email="bioinformatic.seragnoli@gmail.com", 
      role=c("aut", "cre")),
      person("Vincenza", "Solli", email="vincenza.solli2@unibo.it", 
      role=c("aut")))
Description: This package provides a method to refit and correct the diploid
 region in copy number profiles. It uses a clustering algorithm to identify
 pathology-specific normal (diploid) chromosomes and then use their copy number
 signal to refit the whole profile.  The package is composed by three functions:
 DRrefit (the main function), ComputeNormalChromosome and PlotCluster.  
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.2
URL: https://github.com/andrea-poletti-unibo/BOBaFIT
BugReports: https://github.com/andrea-poletti-unibo/BOBaFIT/issues
Imports: dplyr, NbClust, ggplot2, ggbio, grDevices, stats, tidyr,
        GenomicRanges, ggforce, stringr, plyranges, methods, utils,
        magrittr
Suggests: rmarkdown, markdown, BiocStyle, knitr, testthat (>= 3.0.0),
        utils, testthat
Config/testthat/edition: 3
biocViews: CopyNumberVariation, Clustering, Visualization,
        Normalization, Software
Depends: R (>= 2.10)
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/BOBaFIT
git_branch: RELEASE_3_23
git_last_commit: 6e021a1
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 20:58:53 UTC; biocbuild
Author: Andrea Poletti [aut],
  Gaia Mazzocchetti [aut, cre],
  Vincenza Solli [aut]
Maintainer: Gaia Mazzocchetti <bioinformatic.seragnoli@gmail.com>

# Generated by roxygen2: do not edit by hand

export("%>%")
export(DRrefit)
export(DRrefit_plot)
export(PlotChrCluster)
export(Popeye)
export(computeNormalChromosomes)
import(NbClust)
import(ggplot2)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,start)
importFrom(dplyr,arrange)
importFrom(dplyr,desc)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,n)
importFrom(dplyr,rename)
importFrom(dplyr,summarise)
importFrom(ggbio,geom_segment)
importFrom(ggforce,geom_mark_ellipse)
importFrom(grDevices,dev.off)
importFrom(grDevices,jpeg)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,tiff)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(plyranges,join_overlap_intersect)
importFrom(stats,median)
importFrom(stats,weighted.mean)
importFrom(stringr,str_sort)
importFrom(tidyr,"%>%")
importFrom(utils,capture.output)

## ----style, eval=TRUE, echo = FALSE, results = 'asis'----------------------
BiocStyle::latex()

## ----load data, echo=FALSE, message=FALSE, paged.print=FALSE, results='asis'----
library(BOBaFIT)
data("TCGA_BRCA_CN_segments")


## ----computeNormalChromosome, fig.show='hide', include=FALSE, results='hide'----
chr_list <- computeNormalChromosomes(segments = TCGA_BRCA_CN_segments, 
				     tolerance_val = 0.25)

## ----chr_list, results='asis'----------------------------------------------
chr_list

## ----chrlist plot, echo=FALSE, fig.height=10, fig.width=16-----------------
chr_list <- computeNormalChromosomes(segments = TCGA_BRCA_CN_segments, 
				     tolerance_val = 0.25)

## ----DRrefit, eval=FALSE---------------------------------------------------
# results <- DRrefit (segments_chort = TCGA_BRCA_CN_segments, chrlist = chr_list)
# 

## ----DRrefit_test,include=FALSE--------------------------------------------
library(dplyr)
samples <- TCGA_BRCA_CN_segments$ID %>% unique()
sample_test <- samples[1:5]
test <- TCGA_BRCA_CN_segments %>% filter(ID %in% sample_test)
results <- DRrefit (segments_chort = test, chrlist = chr_list)

## ----call output PlotChrCluster code 1, eval=FALSE-------------------------
# results$corrected_segments[1,]

## ----Call segments DRrefit 1, echo=FALSE, paged.print=TRUE-----------------
table <- results$corrected_segments[1,] %>% select(chr,start,end, ID, arm, chrarm,CN,CN_corrected)
knitr::kable(table,digits = 3)


## ----call output PlotChrCluster code 2, eval=FALSE-------------------------
# results$report[1,]

## ----Call report DRrefit 2, echo=FALSE, paged.print=TRUE-------------------
knitr::kable(results$report[1,]) 


## ----DRrefit res-----------------------------------------------------------
corrected_segments <- results$corrected_segments
report <- results$report

## ----DRrefit_plot, eval=FALSE----------------------------------------------
# # the plot is diplayed on the R viewer
# DRrefit_plot(corrected_segments = corrected_segments,
#              DRrefit_report = report,
#              plot_viewer = TRUE,
#              plot_save = FALSE )
# 
# # how to save the plot
# DRrefit_plot(corrected_segments = corrected_segments,
#              DRrefit_report = report,
#              plot_save = TRUE, plot_format = "pdf", plot_path = "/my_path" )

## ----DRrefit_plot 1, echo=FALSE, fig.height=7, fig.width=18----------------
report_s <- report[report$sample == "01428281-1653-4839-b5cf-167bc62eb147",]
seg <- corrected_segments[corrected_segments$ID == "01428281-1653-4839-b5cf-167bc62eb147",]
DRrefit_plot(corrected_segments = seg,
             DRrefit_report = report, 
             plot_viewer = T )

## ----DRrefit_plot 2, echo=FALSE, fig.height=7, fig.width=18----------------
report_s <- report[report$sample == "0941a978-c8aa-467b-8464-9f979d1f0418",]
seg <- corrected_segments[corrected_segments$ID == "0941a978-c8aa-467b-8464-9f979d1f0418",]
DRrefit_plot(corrected_segments = seg,
             DRrefit_report = report, 
             plot_viewer = T )

## ----PlotChrCluster, eval=FALSE--------------------------------------------
# Cluster <- PlotChrCluster(segs = TCGA_BRCA_CN_segments,
#                        clust_method = "ward.D2",
#                        plot_output= TRUE)

## ----PlotChrCluster_test, include=FALSE------------------------------------
Cluster <- PlotChrCluster(segs = test,
                       clust_method = "ward.D2",
                       plot_output= T)


## ----call output PlotChrCluster code, eval=FALSE---------------------------
# head(Cluster$report)
# 
# #select plot table per sample
# head(Cluster$plot_tables$`01428281-1653-4839-b5cf-167bc62eb147`)

## ----Call report PlotChrCluster 3------------------------------------------
knitr::kable(head(Cluster$report)) 

#select plot table per sample
knitr::kable(head(Cluster$plot_tables$`01428281-1653-4839-b5cf-167bc62eb147`)) 

## ----sessionInfo,echo=FALSE------------------------------------------------
sessionInfo()

## ----style, eval=TRUE, echo = FALSE, results = 'asis'----------------------
BiocStyle::latex()

## ----TCGA download, eval=FALSE---------------------------------------------
# BiocManager::install("TCGAbiolinks")
# library(TCGAbiolinks)
# 
# query <- GDCquery(project = "TCGA-BRCA",
#                   data.category = "Copy Number Variation",
#                   data.type = "Copy Number Segment",
#                   sample.type = "Primary Tumor"
#                   )
# 
# #Selecting first 100 samples using the TCGA barcode
# subset <- query[[1]][[1]]
# barcode <- subset$cases[1:100]
# 
# TCGA_BRCA_CN_segments <- GDCquery(project = "TCGA-BRCA",
#                   data.category = "Copy Number Variation",
#                   data.type = "Copy Number Segment",
#                   sample.type = "Primary Tumor",
#                   barcode = barcode
# )
# 
# GDCdownload(TCGA_BRCA_CN_segments, method = "api", files.per.chunk = 50)
# 
# #prepare a data.frame where working
# data <- GDCprepare(TCGA_BRCA_CN_segments, save = TRUE,
# 		   save.filename= "TCGA_BRCA_CN_segments.txt")
# 

## ----Column preparation, eval=FALSE----------------------------------------
# names(data)
# BOBaFIT_names <- c("ID", "chr", "start", "end", "Num_Probes",
# 		   "Segment_Mean","Sample")
# names(data)<- BOBaFIT_names
# names(data)

## ----tcga load, include=FALSE----------------------------------------------
library(BOBaFIT)
data("TCGA_BRCA_CN_segments")
data <- TCGA_BRCA_CN_segments[1:9]


## ----Popeye, echo=TRUE, message=FALSE--------------------------------------
library(BOBaFIT)
segments <- Popeye(data)

## ----Popeye table, echo=FALSE----------------------------------------------
knitr::kable(head(segments))

## ----CN, echo=TRUE---------------------------------------------------------
#When data coming from SNParray platform are used, the user have to apply the
#compression factor in the formula (0.55). In case of WGS/WES data, the
#correction factor is equal to 1.  
compression_factor <- 0.55
segments$CN <- 2^(segments$Segment_Mean/compression_factor + 1)


## ----CN table, echo=FALSE--------------------------------------------------
knitr::kable(head(segments))

## ----sessionInfo,echo=FALSE------------------------------------------------
sessionInfo()

#' DRrefit
#'
#' @description This function refits the diploid region of input copy number profiles (segments - BED file)
#'
#' @param segments_chort data.frame formatted with correct column names
#' @param chrlist list of normal chromosome arms (pathology-specific)
#' @param maxCN threshold of max copy number to consider. By default is 6
#' @param clust_method clustering method. By default is "ward.D2"
#' @param verbose print information about the processes of the function. By default is FALSE

#' @return Return two data frames, one is the DRrefit-corrected segments and the other is the samples report. See the vignette for data frame descriptions.
#' @export
#'
#'
#' @importFrom dplyr filter group_by summarise arrange n desc
#' @import NbClust
#' @importFrom stats median weighted.mean
#' @importFrom GenomicRanges makeGRangesFromDataFrame
#' @importFrom tidyr %>%
#' @importFrom methods is
#' @importFrom utils capture.output
#'
#' @examples
#' data("TCGA_BRCA_CN_segments")
#' 
#' chr_list <- c("10q","11p","12p","19q","1p","21q","2q","3p","4p","4q","6p","6q","7p" )
#' 
#' results <- DRrefit(segments_chort = TCGA_BRCA_CN_segments, 
#'                    chrlist = chr_list)


DRrefit <- function(segments_chort,
                    chrlist,
                    maxCN = 6,
                    clust_method = "ward.D2",
                    verbose= FALSE) {
  
  ward.D2 <- ID <- chrarm <- CN <- width <- chr <- cluster <- CN_corrected <- number <- NULL
  
  OUTPUT <- list()
  
  segments_chort_corrected_list <- list()
  
  report_clustering_list <- list()
  
  segments_chort$CN[segments_chort$CN == Inf] <- maxCN
  
  samples <- segments_chort$ID %>% unique()
  
  if (verbose) {
    
    for (i in seq_along(samples)){
      
      cat("sample n: ",i," - ",samples[i],"\n")
      
      segments <- segments_chort %>%  filter(ID==samples[i])
      
      CN_CHR <- segments %>%
        group_by(chrarm) %>%
        summarise(weighted_mean_CN = weighted.mean(CN, w = width), .groups = 'drop')
      
      
      CN_CHR_values <- CN_CHR$weighted_mean_CN
      
      
      TRY <- try({
        
        
        invisible(capture.output(ClustRes <-NbClust(CN_CHR_values, distance = "euclidean", method = clust_method, index = "all", min.nc = 2, max.nc = 6),  type = ))
        
        
        CLUST_TABLE <- data.frame(chr=CN_CHR$chrarm, cluster=ClustRes$Best.partition, stringsAsFactors = FALSE)
        
        
        CLUST_TABLE_in_list_sorted <- CLUST_TABLE %>%
          filter(chr %in% chrlist ) %>%
          group_by(cluster) %>%
          summarise(number = n()) %>%
          arrange(desc(number))
        
        
        ref_cluster <- CLUST_TABLE_in_list_sorted$cluster[1]
        
        
        
        ref_cluster <- CLUST_TABLE_in_list_sorted$cluster[1]
        
        
        ref_cluster_chrs <- CLUST_TABLE %>% filter(cluster==ref_cluster)
        chrlist_cluster <- ref_cluster_chrs$chr
        
      }, silent = TRUE)
      
      if (is(TRY, "try-error")){
        message("Clustering failed")
        
        new_chrlist <-  chrlist
        
        samp_report <-data.frame(
          sample=samples[i],
          clustering="FAIL",
          ref_clust_chr=paste(chrlist, collapse = ", "),
          num_clust=NA)
        
        
        
        
      } else {
        message("Clustering succeded")
        
        new_chrlist <-  chrlist_cluster
        
        samp_report <-data.frame(
          sample=samples[i],
          clustering="SUCCEDED",
          ref_clust_chr=paste(chrlist_cluster, collapse = ", "),
          num_clust=max(ClustRes$Best.partition))
        
        
      }
      
      
      segments_REF <- segments %>% filter(chrarm %in% new_chrlist)
      
      segments_REF <- segments %>% filter(chrarm %in% new_chrlist)
      
      CN_CHR_REF <- segments_REF %>%
        group_by(chrarm) %>%
        summarise(weighted_mean_CN = weighted.mean(CN, w = width), .groups = 'drop')
      
      real_diploid_region <-  median(CN_CHR_REF$weighted_mean_CN)
      
      correction_factor <- 2 - real_diploid_region
      
      samp_report$correction_factor <- correction_factor
      
      samp_report$correction_class <- ifelse(abs(correction_factor) > 0.5, "REFITTED",
                                             ifelse(abs(correction_factor) <= 0.1, "NO CHANGES", "RECALIBRATED"))
      
      report_clustering_list [[i]]<- samp_report
      
      segments$CN_corrected <- segments$CN + correction_factor
      
      segments$CN_corrected <- ifelse(segments$CN_corrected < 0, 0.001, segments$CN_corrected)
      
      segments_chort_corrected_list[[i]] <- segments
      
    }
    
  } else {
    
    for (i in seq_along(samples)){
      
      
      segments <- segments_chort %>%  filter(ID==samples[i])
      
      CN_CHR <- segments %>%
        group_by(chrarm) %>%
        summarise(weighted_mean_CN = weighted.mean(CN, w = width), .groups = 'drop')
      
      
      CN_CHR_values <- CN_CHR$weighted_mean_CN
      
      
      TRY <- try({
        
        
        invisible(capture.output(ClustRes <- NbClust(CN_CHR_values, distance = "euclidean", method = clust_method, index = "all", min.nc = 2, max.nc = 6), type =c("output", "message")))
        
        
        CLUST_TABLE <- data.frame(chr=CN_CHR$chrarm, cluster=ClustRes$Best.partition, stringsAsFactors = FALSE)
        
        
        CLUST_TABLE_in_list_sorted <- CLUST_TABLE %>%
          filter(chr %in% chrlist ) %>%
          group_by(cluster) %>%
          summarise(number = n()) %>%
          arrange(desc(number))
        
        
        ref_cluster <- CLUST_TABLE_in_list_sorted$cluster[1]
        
        
        
        ref_cluster <- CLUST_TABLE_in_list_sorted$cluster[1]
        
        
        ref_cluster_chrs <- CLUST_TABLE %>% filter(cluster==ref_cluster)
        chrlist_cluster <- ref_cluster_chrs$chr
        
      }, silent = TRUE)
      
      if (is(TRY, "try-error")){
        
        
        new_chrlist <-  chrlist
        
        samp_report <-data.frame(
          sample=samples[i],
          clustering="FAIL",
          ref_clust_chr=paste(chrlist, collapse = ", "),
          num_clust=NA)
        
        
        
        
      } else {
        
        new_chrlist <-  chrlist_cluster
        
        samp_report <-data.frame(
          sample=samples[i],
          clustering="SUCCEDED",
          ref_clust_chr=paste(chrlist_cluster, collapse = ", "),
          num_clust=max(ClustRes$Best.partition))
        
        
      }
      
      
      segments_REF <- segments %>% filter(chrarm %in% new_chrlist)
      
      segments_REF <- segments %>% filter(chrarm %in% new_chrlist)
      
      CN_CHR_REF <- segments_REF %>%
        group_by(chrarm) %>%
        summarise(weighted_mean_CN = weighted.mean(CN, w = width), .groups = 'drop')
      
      real_diploid_region <-  median(CN_CHR_REF$weighted_mean_CN)
      
      correction_factor <- 2 - real_diploid_region
      
      samp_report$correction_factor <- correction_factor
      
      samp_report$correction_class <- ifelse(abs(correction_factor) > 0.5, "REFITTED",
                                             ifelse(abs(correction_factor) <= 0.1, "NO CHANGES", "RECALIBRATED"))
      
      report_clustering_list [[i]]<- samp_report
      
      segments$CN_corrected <- segments$CN + correction_factor
      
      segments$CN_corrected <- ifelse(segments$CN_corrected < 0, 0.001, segments$CN_corrected)
      
      segments_chort_corrected_list[[i]] <- segments
      
    }
    
  }
  
  segments_chort_corrected_df <- Reduce(rbind, segments_chort_corrected_list)
  report_clustering_df <- Reduce(rbind, report_clustering_list)
  
  OUTPUT[["corrected_segments"]] <- segments_chort_corrected_df
  OUTPUT[["report"]] <- report_clustering_df
  
  OUTPUT
}

#' PlotChrCluster
#'
#'
#'@description The function clusters chromosomes based on the copy number (CN) and returns a graph where it is possible to observe the different groups and two data frames (report and plot_table). See the vignette for the data frame descriptions.
#'
#' @param segs data.frame with segments of samples. It must be formatted with correct column names (start, end, ID)
#' @param clust_method clustering method. Default is "ward.D2"
#' @param plot_output Whether to plot refitted profiles (logical)
#' @param plot_viewer Logical parameter. When it is TRUE, the function print the output plot in the R viewer.By default is TRUE.
#' @param plot_save  Logical parameter. When it is TRUE, the function save the plot in the chosen path and format. By default is TRUE.
#' @param plot_format File format for the output plots (accepts "png", "jpg", "pdf", "tiff"). By default is "png" 
#' @param plot_path Path to save output plots.
#' @param verbose print information about the processes of the function. By default is FALSE
#'
#' @return Plot with chromosomes clustered
#' @export
#'
#' @importFrom dplyr filter group_by summarise arrange
#' @import NbClust
#' @import ggplot2
#' @importFrom ggforce geom_mark_ellipse
#' @importFrom stats median weighted.mean
#' @importFrom grDevices png dev.off
#' @importFrom tidyr %>%
#' @importFrom stringr str_sort
#' @importFrom methods is
#' @importFrom utils capture.output
#'
#' @examples
#' data(TCGA_BRCA_CN_segments)
#' Cluster <- PlotChrCluster(segs=TCGA_BRCA_CN_segments, 
#'                          clust_method= "ward.D2", 
#'                          plot_output=FALSE)

PlotChrCluster <- function(segs,
                           clust_method = "ward.D2",
                           plot_output= TRUE,
                           plot_viewer= TRUE,
                           plot_save = FALSE,
                           plot_format = "png",
                           plot_path,
                           verbose= FALSE) {
  report_clustering <- data.frame(sample = character(),
                                  clustering = character(),
                                  num_clust = numeric())
  ID <- chrarm <- CN <- width <- str_sort <- chr <- cluster <- NULL
  
  CLUST_TABLE_LIST <- list()
  
  
  samples <- segs$ID %>% unique()
  if (verbose) {
    for (i in seq_along(samples)) {
      cat("sample n: ", i, " - ", samples[i], "\n")
      
      segments <- segs %>%  filter(ID == samples[i])
      
      CN_CHR <- segments %>%
        group_by(chrarm) %>%
        summarise(weighted_mean_CN = weighted.mean(CN, w = width),
                  .groups = 'drop')
      
      CN_CHR_values <- CN_CHR$weighted_mean_CN
      
      
      TRY <- try({
        capture.output(ClustRes <- NbClust(CN_CHR_values, distance = "euclidean", method = clust_method, index = "all", min.nc = 2, max.nc = 6),  type = c("output", "message"))
        
        CLUST_TABLE <-
          data.frame(
            chr = CN_CHR$chrarm,
            cluster = ClustRes$Best.partition,
            CN = CN_CHR_values,
            stringsAsFactors = FALSE
            
          )
      }, silent = TRUE)
      
      
      if (is(TRY, "try-error")) {
        message("Clustering failed")
        samp_report <- data.frame(sample = samples[i],
                                  clustering = "FAIL",
                                  num_clust = NA)
        
      } else {
        message("Clustering succeded")
        
        samp_report <- data.frame(
          sample = samples[i],
          clustering = "SUCCEDED",
          num_clust = max(ClustRes$Best.partition)
        )
        
        
        CLUST_TABLE$chr <-
          CLUST_TABLE$chr %>% factor(levels = str_sort(CLUST_TABLE$chr, numeric = TRUE),
                                     ordered = TRUE)
        
        CLUST_TABLE$cluster <- paste0("cluster", CLUST_TABLE$cluster)
        
        CLUST_TABLE <- CLUST_TABLE %>% arrange(chr)
        
        if (plot_output) {
          
          gg <- ggplot(CLUST_TABLE, aes(
            x = seq_len(nrow(CLUST_TABLE)),
            y = CN, colour=cluster)) +
            ylim(-0.5, max(CLUST_TABLE$CN)) +
            geom_mark_ellipse( aes(fill = cluster)) +
            geom_hline(yintercept = 2, alpha = 0.5) +
            geom_point(size = 2) +
            geom_label(aes(label = chr), nudge_y = 0.1) +
            ggtitle(samples[i])+
            xlab("Chromosomal arms")+ 
            ylab("Copy Number")
          
          options(ggplot2. = "viridis")
          
          if(plot_save){
            
            if ( plot_format=="png" ) { png( paste0(plot_path, samples[i],"_PlotChrCluster.png"), width = 16, height = 4, units = "in", res = 300 ) 
            } else if(plot_format=="tiff") { tiff( paste0(plot_path, samples[i],"_PlotChrCluster.tif"), width = 16, height = 4, units = "in", res = 300 )
            } else if(plot_format=="pdf") { pdf( file = paste0(plot_path, samples[i],"_PlotChrCluster.pdf"), width = 16, height = 4 )
            } else if(plot_format == "jpg") { jpeg( paste0(plot_path, samples[i],"_PlotChrCluster.jpg"), width = 16, height = 4, units = "in", res = 300 ) 
            } else { message("wrong plot format") 
              break}
            
            print(gg)
            
            dev.off()
          }
          
          if(plot_viewer){
            print(gg)
          }
          
        }
        
        
        report_clustering <- rbind(report_clustering, samp_report)
        
        CLUST_TABLE_LIST [[samples[i]]] <- CLUST_TABLE
        
      }
    }} else {
      
      for (i in seq_along(samples)) {
        
        segments <- segs %>%  filter(ID == samples[i])
        
        CN_CHR <- segments %>%
          group_by(chrarm) %>%
          summarise(weighted_mean_CN = weighted.mean(CN, w = width),
                    .groups = 'drop')
        
        CN_CHR_values <- CN_CHR$weighted_mean_CN
        
        
        TRY <- try({
          capture.output(ClustRes <- NbClust(CN_CHR_values, distance = "euclidean", method = clust_method, index = "all", min.nc = 2, max.nc = 6),  type = c("output", "message"))
          
          CLUST_TABLE <-
            data.frame(
              chr = CN_CHR$chrarm,
              cluster = ClustRes$Best.partition,
              CN = CN_CHR_values,
              stringsAsFactors = FALSE
              
            )
        }, silent = TRUE)
        
        
        if (is(TRY, "try-error")) {
          samp_report <- data.frame(sample = samples[i],
                                    clustering = "FAIL",
                                    num_clust = NA)
          
        } else {
          
          samp_report <- data.frame(
            sample = samples[i],
            clustering = "SUCCEDED",
            num_clust = max(ClustRes$Best.partition)
          )
          
          
          CLUST_TABLE$chr <-
            CLUST_TABLE$chr %>% factor(levels = str_sort(CLUST_TABLE$chr, numeric = TRUE),
                                       ordered = TRUE)
          
          CLUST_TABLE$cluster <- paste0("cluster", CLUST_TABLE$cluster)
          
          CLUST_TABLE <- CLUST_TABLE %>% arrange(chr)
          
          if (plot_output) {
            
            gg <- ggplot(CLUST_TABLE, aes(
              x = seq_len(nrow(CLUST_TABLE)),
              y = CN, colour=cluster)) +
              ylim(-0.5, max(CLUST_TABLE$CN)) +
              geom_mark_ellipse( aes(fill = cluster)) +
              geom_hline(yintercept = 2, alpha = 0.5) +
              geom_point(size = 2) +
              geom_label(aes(label = chr), nudge_y = 0.1) +
              ggtitle(samples[i])+
              xlab("Chromosomal arms")+ 
              ylab("Copy Number")
            
            
            options(ggplot2. = "viridis")
            
            if(plot_save){
              
              if ( plot_format=="png" ) { png( paste0(plot_path, samples[i],"_PlotChrCluster.png"), width = 16, height = 4, units = "in", res = 300 ) 
              } else if(plot_format=="tiff") { tiff( paste0(plot_path, samples[i],"_PlotChrCluster.tif"), width = 16, height = 4, units = "in", res = 300 )
              } else if(plot_format=="pdf") { pdf( file = paste0(plot_path, samples[i],"_PlotChrCluster.pdf"), width = 16, height = 4 )
              } else if(plot_format == "jpg") { jpeg( paste0(plot_path, samples[i],"_PlotChrCluster.jpg"), width = 16, height = 4, units = "in", res = 300 ) 
              } else { message("wrong plot format") 
                break}
              
              print(gg)
              
              dev.off()
            }
            
            if(plot_viewer){
              print(gg)
            }
            
          }
          
          
          
          
          report_clustering <- rbind(report_clustering, samp_report)
          
          CLUST_TABLE_LIST [[samples[i]]] <- CLUST_TABLE
          
        }} }
  
  OUTPUT <-
    list(report = report_clustering , plot_tables = CLUST_TABLE_LIST)
  OUTPUT
}

#' computeNormalChromosomes
#'
#' @description This function compute the DRrefits' input "chromosome list". It is a vector that contains the chromosomal arms considered "normal" in the cohort of samples tested (BED file), under a specific tolerance value
#'
#' @param segments data.frame formatted with correct column names
#' @param tolerance_val decimal value of alteration frequency. By default is 0.15
#' @param maxCN threshold of max copy number to consider. By default is 6
#' @param min_threshold minimum threshold to define a normal CN. By default is 1.60
#' @param max_threshold maximum threshold to define a normal CN. By default is 2.40
#' @param verbose print information about the processes of the function. By default is FALSE
#'
#' @importFrom dplyr filter group_by summarise arrange
#' @importFrom tidyr %>%
#' @importFrom stats weighted.mean
#' @import ggplot2
#' @importFrom stringr str_sort
#'
#'
#' @return vector with chromosome names and plot with the alteration rate of each chromosomal arms
#' @export
#'
#' @examples
#' data("TCGA_BRCA_CN_segments")
#' chr_list <- computeNormalChromosomes(segments = TCGA_BRCA_CN_segments)



computeNormalChromosomes <- function(segments,
                                     tolerance_val = 0.15,
                                     maxCN = 6,
                                     min_threshold = 1.60,
                                     max_threshold = 2.40,
                                     verbose = FALSE) {
  ID <-
    arm <-
    chrarm <-
    weighted.mean <-
    CN <- width <- weighted_mean_CN <- alteration_rate <- NULL
  
  samples <- segments$ID %>% unique()
  segments$CN[segments$CN > 6] <- maxCN
  
  all_chromosome <- data.frame()
  
  if (verbose) {  
    for (i in seq_along(samples)) {
      message(i, " - ", samples[i])
      segments_sample <- segments %>% filter(ID == samples[i])
      
      CN_CHR <- segments_sample %>%
        group_by(chrarm) %>%
        summarise(weighted_mean_CN = weighted.mean(CN, w = width),
                  .groups = 'drop_last') %>%
        filter(weighted_mean_CN <= max_threshold &
                 weighted_mean_CN >= min_threshold)
      
      all_chromosome <- rbind(all_chromosome, CN_CHR)
      
    }
  } else {
    for (i in seq_along(samples)) {
      segments_sample <- segments %>% filter(ID == samples[i])
      
      CN_CHR <- segments_sample %>%
        group_by(chrarm) %>%
        summarise(weighted_mean_CN = weighted.mean(CN, w = width),
                  .groups = 'drop_last') %>%
        filter(weighted_mean_CN <= max_threshold &
                 weighted_mean_CN >= min_threshold)
      
      all_chromosome <- rbind(all_chromosome, CN_CHR)
    }
  }
  
  result <- (table(all_chromosome$chrarm) / length(samples))
  result_filtered <- result[result > 1 - tolerance_val]
  
  
  
  df <-
    data.frame(
      arm = names(result),
      alteration_rate = as.numeric(1 - result),
      observations = table(all_chromosome$chrarm)
    )
  df$arm <-
    df$arm %>% factor(levels = str_sort(df$arm, numeric = TRUE))
  print(
    ggplot(df, aes(x = arm, y = alteration_rate)) +
      geom_bar(stat = "identity", aes(fill = alteration_rate > tolerance_val)) +
      geom_hline(
        yintercept = tolerance_val,
        colour = "black",
        linetype = 2
      ) +
      geom_text(aes(
        label = round(alteration_rate, 2), vjust = 1.4
      )) +
      scale_fill_manual(values = c("#1E90FF", "#FF4040")) +
      theme(legend.position = "bottom") +
      labs(y = "Alteration Rate (%)", x = "Chromosomal arm")
  )
  
  names(result_filtered)
  
}

#' Popeye
#' @description The function assign the chromosomal arm to each segment.
#'
#' @param segments data.frame formatted with correct column names (see package vignette)
#'
#' @return Return a data frame containg segments with the arm annotation.
#' @export
#' 
#' @importFrom tidyr %>%
#' @importFrom GenomicRanges makeGRangesFromDataFrame seqnames start end
#' @importFrom dplyr filter arrange rename
#' @importFrom plyranges join_overlap_intersect
#' 
#' 
#' @examples
#' data("TCGA_BRCA_CN_segments")
#' data <- TCGA_BRCA_CN_segments[1:9] #as it already presents the arm column
#' data_annotated <- Popeye(segments = data)

Popeye <- function(segments) {
  chr <- ID <- NULL
  
  segments <- segments %>% filter(!chr %in% c("X", "Y"))
  
  chrtab <- BOBaFIT:::chrtab

  chrtabGR <- makeGRangesFromDataFrame(chrtab) 
  
  segmentsGR <- makeGRangesFromDataFrame(segments, keep.extra.columns = TRUE)
  
  segmentsGR_noCentromeres <- join_overlap_intersect(chrtabGR, segmentsGR)

  segments_clean <- segmentsGR_noCentromeres %>% as.data.frame()
  
  arms <- chrtab$chrarm %>% unique()
  
  annot_data <- data.frame()
  i=1
  for( i in seq_along(arms)){
    
    message(i, " - arm: ", arms[i])
    
    chr_sel <- chrtab$chr[i]
    arm_sel <- chrtab$arm[i]
    chrarm_sel <- chrtab$chrarm[i]
    start_sel <- chrtab$start[i]
    end_sel <- chrtab$end[i]
    
    data_f <- segments_clean %>% filter(seqnames==chr_sel, start >= start_sel, end<= end_sel)
    
    
    data_f$arm <- arm_sel
    data_f$chrarm <- chrarm_sel
    
    annot_data <- rbind(annot_data, data_f)
    
  }

  annot_data_sort <- annot_data %>% arrange(ID, seqnames, start)
  
  OUTPUT <- annot_data_sort %>% rename("chr"="seqnames")
  OUTPUT
  
}

#' DRrefit_plot
#'
#' @description The function plot the copy number profile before and after DRrefit recalibration 
#'
#' @param corrected_segments DRrefit output dataframe. 
#' @param DRrefit_report  DRrefit output dataframe.
#' @param plot_viewer Logical parameter. When it is TRUE, the function print the output plot in the R viewer.By default is FALSE.
#' @param plot_save  Logical parameter. When it is TRUE, the function save the plot in the chosen path and format. By default is FALSE.
#' @param plot_format File format for the output plots (accepts "png", "jpg", "pdf", "tiff"). By default is "png" 
#' @param plot_path Path to save output plots.
#'
#' @return Return the sample copy number profile before and after DRrefit recalibration. The function can output the figure in the R viewer on save it in a specific path.
#' @export
#' 
#' @importFrom grDevices png tiff pdf jpeg dev.off
#' @import ggplot2
#' @importFrom ggbio geom_segment
#' @importFrom GenomicRanges makeGRangesFromDataFrame 
#' @importFrom tidyr %>%
#' @importFrom dplyr filter 
#'
#' @examples
#' data("TCGA_BRCA_CN_segments")
#' 
#' chr_list <- c("10q","11p","12p","19q","1p","21q","2q","3p","4p","4q","6p","6q","7p" )
#' 
#' results <- DRrefit(segments_chort = TCGA_BRCA_CN_segments, chrlist = chr_list)
#'                    
#' my_segments <- results$corrected_segments
#' my_report <- results$report
#' 
#' DRrefit_plot(corrected_segments = my_segments,
#'              DRrefit_report = my_report, 
#'              plot_viewer= FALSE, 
#'              plot_save = FALSE)
#' 
#' 

DRrefit_plot <- function(corrected_segments,
                         DRrefit_report,
                         plot_viewer= F,
                         plot_save = F,
                         plot_format = "png",
                         plot_path
) {
  
  ID <- CN <- CN_corrected <-  NULL
  
  samples <- unique(corrected_segments$ID)
  
  for (i in seq_along(samples)) {
    
    segments <- corrected_segments %>%  filter(ID==samples[i])
    
    report_samp <- DRrefit_report %>%  filter(sample== samples[i])
    
    Granges_segments <- makeGRangesFromDataFrame(segments, keep.extra.columns = TRUE)
    
    chromosome_list <- report_samp$ref_clust_chr
    correction_factor <- report_samp$correction_factor
    
    
    gg <- ggplot() +
      ggtitle(paste0("Sample name: ",samples[i]),
              subtitle = paste0("Correction factor= ", correction_factor %>% round(3)," / diploid chrs: ", chromosome_list)) +
      geom_segment(data= Granges_segments,  stat="identity", aes(y= CN, color="old CN"), size = 1.2, alpha=0.7) +
      geom_segment(data = Granges_segments, stat="identity", aes(y= CN_corrected, color="corrected CN"), size = 1.2, alpha=0.7) +
      ylim(0,5) +
      facet_grid( ~seqnames, scales = "free", space = "free", margins = FALSE, switch="x") +
      theme_bw() +
      theme(panel.spacing=unit(.05, "lines"), 
            panel.border = element_rect(color = "black", fill = NA, size = 0.2), 
            axis.text.x = element_text(size=6, angle = 90, vjust = 0.5, hjust=1),
            strip.background = element_rect(color="black", fill="grey90", linetype="solid"),
            strip.text.x = element_text(face = "bold", size=8),
            legend.position="bottom") +
      scale_x_continuous(labels = function(x) format(x/1000000, scientific = FALSE), 
                         breaks = seq(0, 300*10^6,  by = 25*10^6),
                         expand = c(0, 0), 
                         limits = c(NA, NA))+
      geom_hline(yintercept = 2, linetype=3) +
      xlab("Mbp (genomic position)")+
      if(abs(correction_factor) > 0.1 ){
        scale_color_manual(values=c("green4", "red3")) 
      } else {
        scale_color_manual(values=c("orange", "red3"))
      }
    
    
    
    if(plot_save){
      
      if ( plot_format=="png" ) { png( paste0(plot_path, samples[i],".png"), width = 16, height = 4, units = "in", res = 300 ) 
      } else if(plot_format=="tiff") { tiff( paste0(plot_path, samples[i],".tif"), width = 16, height = 4, units = "in", res = 300 )
      } else if(plot_format=="pdf") { pdf( file = paste0(plot_path, samples[i],".pdf"), width = 16, height = 4 )
      } else if(plot_format == "jpg") { jpeg( paste0(plot_path, samples[i],".jpg"), width = 16, height = 4, units = "in", res = 300 ) 
      } else { message("wrong plot format") 
        break}
      
      print(gg)
      
      
      dev.off()
    }
    
    if(plot_viewer){
      print(gg)
    }
    
    
  }
}

---
title: "BOBaFIT"
author: 
- name: Gaia Mazzocchetti 
  affiliation: University of Bologna - DIMES 
  email: bioinformatics.seragnoli@gmail.com 
date: "`r Sys.Date()`"
package: BOBaFIT 
output: BiocStyle::html_document
abstract: |
  Good practices for BOBaFIT package
vignette: |
  %\VignetteIndexEntry{BOBaFIT.Rmd}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: references.bib
---

```{r style, eval=TRUE, echo = FALSE, results = 'asis'}
BiocStyle::latex()
```

# Introduction

The R package BoBafit is composed of four functions which allow the refit and the recalibration of copy number profile of tumor sample. In particular, the package was built to check, and possibly correct, the diploid regions. The wrong diploid region is a phenomenon that very often affects the profiles of samples with a very complex karyotype.

The principal and refitting function was named `DRrefit`, which - throughout a chromosome clustering method and a list of unaltered chromosomes (chromosome list) - recalibrates the copy number values. BoBafit also contains three secondary functions: `ComputeNormalChromosome`, which generates the chromosome list; `PlotChrCluster`, where is possible to visualize the cluster; and `Popeye`, which affixes its chromosomal arm to each segment (see in "Data Preparation" vignette).

# Data

The package checks the diploid region assessment working on pre-estimated segment information, as the copy number and their position. We included a data set `TCGA_BRCA_CN_segments` where are showed all the information necessary. The data correspond to segments about 100 breast tumors samples obtained by the project TCGA-BRCA [@Tomczak2015]. In the "Data Preparation" vingnette is shown how we download and prepare the dataset for the following analysis.

```{r load data, echo=FALSE, message=FALSE, paged.print=FALSE, results='asis'}
library(BOBaFIT)
data("TCGA_BRCA_CN_segments")

```

# BOBaFIT Workflow

Once the dataset has been prepared, the next step is to generate the chromosome list. The chromosome list is a vector containing all chromosomal arm which are the least affected by SCNAs in the tumor analyzed. Together with the clustering, the chromosome list is one the operating principles to rewrite the diploid region. The list can be manually created or by using the function `ComputeNormalChromosome`*.* We suggest these two sequential steps to allow the right refit and recalibration of sample's diploid region:

1.  `ComputeNormalChromosome()`

2.  `DRrefit()`

Here we performed this analysis workflow on the dataset `TCGA_BRCA_CN_segments` described above.

## ComputeNormalChromosome

The chromosome list is a vector specific for each tumor (type and subtype) . The chromosome arms included in this list must be selected based on how many CNA events they are subject to and how many times their CN falls into a "diploid range". According to this principle, *ComputeNormalChromosome* write the chromosome list. The function allows to set the chromosomal alteration rate (`tolerance_val`), which corresponds to a minimum percentage of alterations that one wants to tolerate per arm.

With a little dataframe (less than 200 samples), we suggest an alteration rate of 5% (0.5) ; on the contrary, With a big dataframe (about 1000 samples), we suggest as maximum rate 20-25% (0.20-0.25) . The function input is a sample cohort with their segments.

Here we performed the function in the data set `TCGA_BRCA_CN_segments`, using an alteration rate of 25%.

```{r computeNormalChromosome, fig.show='hide', include=FALSE, results='hide'}
chr_list <- computeNormalChromosomes(segments = TCGA_BRCA_CN_segments, 
				     tolerance_val = 0.25)
```

```{r chr_list, results='asis'}
chr_list
```

Storing the result in the variable `chr_list`, it will be a vector containing the chromosomal arms which present an alteration rate under the indicated `tolerance_val` value.

The function also plots in the Viewer a histogram where is possible observe the chromosomal alteration rate of each chromosomal arms and which one have been selected in the chromosome list (blue bars). The tolerance value has been set at 0.25 (dotted line).


```{r chrlist plot, echo=FALSE, fig.height=10, fig.width=16}
chr_list <- computeNormalChromosomes(segments = TCGA_BRCA_CN_segments, 
				     tolerance_val = 0.25)
```

## DRrefit

To create a tumor-specific method that refit and recalibrate the tumor copy number profile, we developed the function `DRrefit`. It uses as input the sample's segments - **cohort** or **single sample**-, and the chromosome list.

As said before, `DRrefit` estimates the right diploid region using two operating principle: a clustering function `NbClust` [@charrad2014], which allow to estimete the best number of cluster in the sample, and the chromosome list. The clustering method can be sets with the argument `clust_method`. The options are: "ward.D", "ward.D2", "single", "complete", "average", "mcquitty", "median", "centroid" and "kmeans".

In this example, the `TCGA_BRCA_CN_segments`data table and the `chr_list` previously generated are used. The default value of `maxCN` (6) and `clust_method` (ward.d2) are used.

```{r DRrefit, eval=FALSE}
results <- DRrefit (segments_chort = TCGA_BRCA_CN_segments, chrlist = chr_list)

```

```{r DRrefit_test,include=FALSE}
library(dplyr)
samples <- TCGA_BRCA_CN_segments$ID %>% unique()
sample_test <- samples[1:5]
test <- TCGA_BRCA_CN_segments %>% filter(ID %in% sample_test)
results <- DRrefit (segments_chort = test, chrlist = chr_list)
```

### The Dataframes

-   The data frame `corrected_segments` reports the CN corrected of the segments by the correction factor (CR) - value estimated for each sample by the function to correct the diploid region-
```{r call output PlotChrCluster code 1, eval=FALSE}
results$corrected_segments[1,]
```

```{r Call segments DRrefit 1, echo=FALSE, paged.print=TRUE}
table <- results$corrected_segments[1,] %>% select(chr,start,end, ID, arm, chrarm,CN,CN_corrected)
knitr::kable(table,digits = 3)

```

It is similar to the input one and report the new CN value of each segment calculated by DRrefit (`CN_corrected`).

-   The data frame `report` contains all the information about the clustering as the outcome,the number of clusters found in that sample, the chromosome list and the CR used for the adjustment of the diploid region. Sample are also divided in three classes, based on their CR: No changes (CR\<0.1); Recalibrated (0.1\<CR\<0.5); Refitted (CR \>0.5). The class label is also reported in the data frame `report`.
```{r call output PlotChrCluster code 2, eval=FALSE}
results$report[1,] 
```

```{r Call report DRrefit 2, echo=FALSE, paged.print=TRUE}
knitr::kable(results$report[1,]) 

```

When the column `clustering` reports FAIL, it indicates that , NbClust fails the chromosome clustering in that sample. In this case, the sample will not present clusters, so the input chromosome list will be kept as reference . When the column `clustering` reports SUCCED, NbClust succeeds and and the new chromosome list is chosen. The chromosome list used for each sample are all reported in the column `ref_clust_chr`.

## DRrefit_plot

Thanks to the function `DRrefit_plot` is possible appreciate the CN profile before and after the correction made by`DRrefit`. It makes a plot for each sample with the old and new segments positions. The x-axes represent the chromosomes with their genomic position, and the y-axes the copy number value. Above the plot are reported the sample name, the CR and the chromosomal arm used as reference to estimate the new diploid region.

```{r DRrefit res}
corrected_segments <- results$corrected_segments
report <- results$report
```
```{r DRrefit_plot, eval=FALSE}
# the plot is diplayed on the R viewer
DRrefit_plot(corrected_segments = corrected_segments,
             DRrefit_report = report, 
             plot_viewer = TRUE, 
             plot_save = FALSE )

# how to save the plot 
DRrefit_plot(corrected_segments = corrected_segments,
             DRrefit_report = report, 
             plot_save = TRUE, plot_format = "pdf", plot_path = "/my_path" )
```

Based on the CR value two plots can be displayed:

-   CR ≤ 0.1: the new segment and the old segments are orange and red colored, respectively;

```{r DRrefit_plot 1, echo=FALSE, fig.height=7, fig.width=18}
report_s <- report[report$sample == "01428281-1653-4839-b5cf-167bc62eb147",]
seg <- corrected_segments[corrected_segments$ID == "01428281-1653-4839-b5cf-167bc62eb147",]
DRrefit_plot(corrected_segments = seg,
             DRrefit_report = report, 
             plot_viewer = T )
```


-   CR \> 0.1: the new segment and the old segments are green and red colored, respectively;

```{r DRrefit_plot 2, echo=FALSE, fig.height=7, fig.width=18}
report_s <- report[report$sample == "0941a978-c8aa-467b-8464-9f979d1f0418",]
seg <- corrected_segments[corrected_segments$ID == "0941a978-c8aa-467b-8464-9f979d1f0418",]
DRrefit_plot(corrected_segments = seg,
             DRrefit_report = report, 
             plot_viewer = T )
```


# PlotChrCluster

Another accessory function is `PlotChrCluster`. It can be used to visualize the chromosomal cluster in a single sample or in a sample cohort. The input data is always a .tsv file, as the data frame `TCGA_BRCA_CN_segments`. The option of `clust_method` argument are the same of `DRrefit`("ward.D", "ward.D2", "single", "complete", "average", "mcquitty", "median", "centroid" and "kmeans").

```{r PlotChrCluster, eval=FALSE}
Cluster <- PlotChrCluster(segs = TCGA_BRCA_CN_segments,
                       clust_method = "ward.D2",
                       plot_output= TRUE)
```

```{r PlotChrCluster_test, include=FALSE}
Cluster <- PlotChrCluster(segs = test,
                       clust_method = "ward.D2",
                       plot_output= T)

```

We suggest to store the output on a variable (in this example we use `Cluster`) to view and possibly save the data frame generated. The `PlotCuster` will automatically save the plot in the folder indicated by the variable `path` of the argument `plot_path`.

In the `PlotChrCluster` plot, the chromosomal arms are labeled and colored according to the cluster they belong to. The y-axis reports the arm CN.


![](figure/SAMPLE_100_PlotChrCluster.png){style="text-align: center;" width="100%"}

## The Dataframes

The outputs `report` summarizes the outcome of clustering for each sample (fail or succeeded, the number of clusters), similar to DRrefit report output. The second output, `plot tables`, is a list of data frames (one per sample) and reports in which clustering the chromosomes of the sample have been placed.
```{r call output PlotChrCluster code, eval=FALSE}
head(Cluster$report) 

#select plot table per sample
head(Cluster$plot_tables$`01428281-1653-4839-b5cf-167bc62eb147`) 
```

```{r Call report PlotChrCluster 3}
knitr::kable(head(Cluster$report)) 

#select plot table per sample
knitr::kable(head(Cluster$plot_tables$`01428281-1653-4839-b5cf-167bc62eb147`)) 
```

# Session info {.unnumbered}

```{r sessionInfo,echo=FALSE}
sessionInfo()
```

# Reference {.unnumbered}

---
title: "Data preparation using TCGA-BRCA database" 
author: 
- name: Gaia Mazzocchetti 
  affiliation: University of Bologna - DIMES 
  email: bioinformatics.seragnoli@gmail.com 
date: "`r Sys.Date()`"
package: BOBaFIT 
output: 
 BiocStyle::html_document 
abstract: | 
 Best practise to download and prepare TCGA-BRCA dataset for BOBaFIT's analysis 
vignette: |
 %\VignetteIndexEntry{Data preparation using TCGA-BRCA database.Rmd} 
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8} 
bibliography: references.bib 
---

```{r style, eval=TRUE, echo = FALSE, results = 'asis'}
BiocStyle::latex()
```

# Introduction

The data preparation is an important step before the BOBaFIT analysis. In this vignete we will explain how to download the TCGA-BRCA dataset[@Tomczak2015] the package TCGAbiolink [@Colaprico2016] and how to add information like chromosomal arm and CN value of each segments, which operating principle of the package. Further, here we show the column names of the input file for all the BOBaFIT function.

# Download from TCGA

To download the TCGCA-BRCA[@Tomczak2015], we used the R package TCGAbiolinks [@Colaprico2016]and we construct the query. The query includs Breast Cancer samples analyzed by SNParray method (GenomeWide_SNP6), obtaining their Copy Number (CN) profile.

```{r TCGA download, eval=FALSE}
BiocManager::install("TCGAbiolinks")
library(TCGAbiolinks)

query <- GDCquery(project = "TCGA-BRCA",
                  data.category = "Copy Number Variation",
                  data.type = "Copy Number Segment",
                  sample.type = "Primary Tumor"
                  )

#Selecting first 100 samples using the TCGA barcode 
subset <- query[[1]][[1]]
barcode <- subset$cases[1:100]

TCGA_BRCA_CN_segments <- GDCquery(project = "TCGA-BRCA",
                  data.category = "Copy Number Variation",
                  data.type = "Copy Number Segment",
                  sample.type = "Primary Tumor",
                  barcode = barcode
)

GDCdownload(TCGA_BRCA_CN_segments, method = "api", files.per.chunk = 50)

#prepare a data.frame where working
data <- GDCprepare(TCGA_BRCA_CN_segments, save = TRUE, 
		   save.filename= "TCGA_BRCA_CN_segments.txt")
 
```

In the last step, a dataframe with the segments of all samples is prepared. However some information are missing, so the dataset is not ready as BOBaFIT input.

# Columns preparation

Further, here we show the **column names** of the input file for all the BOBaFIT function.

```{r Column preparation, eval=FALSE}
names(data)
BOBaFIT_names <- c("ID", "chr", "start", "end", "Num_Probes", 
		   "Segment_Mean","Sample")
names(data)<- BOBaFIT_names
names(data)
```

# Assign the chromosome arm with Popeye

The arm column is an very important information that support the diploid region check of`DRrefit` and the chromosome list computation of `ComputeNormalChromosome`. As it is lacking in the TCGA-BRCAdataset, the function `Popeye`has been specially designed to calculate which chromosomal arm the segment belongs to. Thanks to this algorithm, not only the TCGA-BRCA dataset, but any database you want to analyze can be analyzed by any function of BOBaFIT.

```{r tcga load, include=FALSE}
library(BOBaFIT)
data("TCGA_BRCA_CN_segments")
data <- TCGA_BRCA_CN_segments[1:9]

```

```{r Popeye, echo=TRUE, message=FALSE}
library(BOBaFIT)
segments <- Popeye(data)
```
```{r Popeye table, echo=FALSE}
knitr::kable(head(segments))
```

# Calculation of the Copy Number

The last step is the computation of the copy number value from the "`Segment_Mean`" column (logR), with the following formula. At this point the data is ready to be analyzed by the whole package.

```{r CN, echo=TRUE}
#When data coming from SNParray platform are used, the user have to apply the
#compression factor in the formula (0.55). In case of WGS/WES data, the
#correction factor is equal to 1.  
compression_factor <- 0.55
segments$CN <- 2^(segments$Segment_Mean/compression_factor + 1)

```
```{r CN table, echo=FALSE}
knitr::kable(head(segments))
```

# Session info {.unnumbered}

```{r sessionInfo,echo=FALSE }
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeNormalChromosomes.R
\name{computeNormalChromosomes}
\alias{computeNormalChromosomes}
\title{computeNormalChromosomes}
\usage{
computeNormalChromosomes(
  segments,
  tolerance_val = 0.15,
  maxCN = 6,
  min_threshold = 1.6,
  max_threshold = 2.4,
  verbose = FALSE
)
}
\arguments{
\item{segments}{data.frame formatted with correct column names}

\item{tolerance_val}{decimal value of alteration frequency. By default is 0.15}

\item{maxCN}{threshold of max copy number to consider. By default is 6}

\item{min_threshold}{minimum threshold to define a normal CN. By default is 1.60}

\item{max_threshold}{maximum threshold to define a normal CN. By default is 2.40}

\item{verbose}{print information about the processes of the function. By default is FALSE}
}
\value{
vector with chromosome names and plot with the alteration rate of each chromosomal arms
}
\description{
This function compute the DRrefits' input "chromosome list". It is a vector that contains the chromosomal arms considered "normal" in the cohort of samples tested (BED file), under a specific tolerance value
}
\examples{
data("TCGA_BRCA_CN_segments")
chr_list <- computeNormalChromosomes(segments = TCGA_BRCA_CN_segments)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DRrefit_plot.R
\name{DRrefit_plot}
\alias{DRrefit_plot}
\title{DRrefit_plot}
\usage{
DRrefit_plot(
  corrected_segments,
  DRrefit_report,
  plot_viewer = F,
  plot_save = F,
  plot_format = "png",
  plot_path
)
}
\arguments{
\item{corrected_segments}{DRrefit output dataframe.}

\item{DRrefit_report}{DRrefit output dataframe.}

\item{plot_viewer}{Logical parameter. When it is TRUE, the function print the output plot in the R viewer.By default is FALSE.}

\item{plot_save}{Logical parameter. When it is TRUE, the function save the plot in the chosen path and format. By default is FALSE.}

\item{plot_format}{File format for the output plots (accepts "png", "jpg", "pdf", "tiff"). By default is "png"}

\item{plot_path}{Path to save output plots.}
}
\value{
Return the sample copy number profile before and after DRrefit recalibration. The function can output the figure in the R viewer on save it in a specific path.
}
\description{
The function plot the copy number profile before and after DRrefit recalibration
}
\examples{
data("TCGA_BRCA_CN_segments")

chr_list <- c("10q","11p","12p","19q","1p","21q","2q","3p","4p","4q","6p","6q","7p" )

results <- DRrefit(segments_chort = TCGA_BRCA_CN_segments, chrlist = chr_list)
                   
my_segments <- results$corrected_segments
my_report <- results$report

DRrefit_plot(corrected_segments = my_segments,
             DRrefit_report = my_report, 
             plot_viewer= FALSE, 
             plot_save = FALSE)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TCGA_BRCA_CN_segments.R
\docType{data}
\name{TCGA_BRCA_CN_segments}
\alias{TCGA_BRCA_CN_segments}
\title{Segments of 100 Breast Cancer samples, downloaded from TCGA-BRCA.}
\format{
A data frame with79,607 rows and 12 variables:
\describe{
  \item{chr}{Chrosome which the segment belong}
  \item{start}{Starting point of the segment, in Mb}
  \item{end}{Ending point of the segment, in Mb}
  \item{width}{Width of the segment, in Mb}
  \item{strand}{Strand of the segment}
  \item{ID}{Sample name}
  \item{Num_Probes}{Probes involved}
  \item{Segment_Mean}{LogR of the segments}
  \item{Sample}{Barcode of tCGA-BRCA database}
  \item{arm}{Arm information, p o q}
  \item{chrarm}{Chromosomal arm which the segment belong}
  \item{CN}{Segments Copy Number value obtained by the logR }
}
}
\source{
\url{https://portal.gdc.cancer.gov/projects/TCGA-BRCA}
}
\usage{
TCGA_BRCA_CN_segments
}
\description{
Segments of 100 Breast Cancer samples, downloaded from TCGA-BRCA.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DRrefit.R
\name{DRrefit}
\alias{DRrefit}
\title{DRrefit}
\usage{
DRrefit(
  segments_chort,
  chrlist,
  maxCN = 6,
  clust_method = "ward.D2",
  verbose = FALSE
)
}
\arguments{
\item{segments_chort}{data.frame formatted with correct column names}

\item{chrlist}{list of normal chromosome arms (pathology-specific)}

\item{maxCN}{threshold of max copy number to consider. By default is 6}

\item{clust_method}{clustering method. By default is "ward.D2"}

\item{verbose}{print information about the processes of the function. By default is FALSE}
}
\value{
Return two data frames, one is the DRrefit-corrected segments and the other is the samples report. See the vignette for data frame descriptions.
}
\description{
This function refits the diploid region of input copy number profiles (segments - BED file)
}
\examples{
data("TCGA_BRCA_CN_segments")

chr_list <- c("10q","11p","12p","19q","1p","21q","2q","3p","4p","4q","6p","6q","7p" )

results <- DRrefit(segments_chort = TCGA_BRCA_CN_segments, 
                   chrlist = chr_list)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Popeye.R
\name{Popeye}
\alias{Popeye}
\title{Popeye}
\usage{
Popeye(segments)
}
\arguments{
\item{segments}{data.frame formatted with correct column names (see package vignette)}
}
\value{
Return a data frame containg segments with the arm annotation.
}
\description{
The function assign the chromosomal arm to each segment.
}
\examples{
data("TCGA_BRCA_CN_segments")
data <- TCGA_BRCA_CN_segments[1:9] #as it already presents the arm column
data_annotated <- Popeye(segments = data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotChrCluster.R
\name{PlotChrCluster}
\alias{PlotChrCluster}
\title{PlotChrCluster}
\usage{
PlotChrCluster(
  segs,
  clust_method = "ward.D2",
  plot_output = TRUE,
  plot_viewer = TRUE,
  plot_save = FALSE,
  plot_format = "png",
  plot_path,
  verbose = FALSE
)
}
\arguments{
\item{segs}{data.frame with segments of samples. It must be formatted with correct column names (start, end, ID)}

\item{clust_method}{clustering method. Default is "ward.D2"}

\item{plot_output}{Whether to plot refitted profiles (logical)}

\item{plot_viewer}{Logical parameter. When it is TRUE, the function print the output plot in the R viewer.By default is TRUE.}

\item{plot_save}{Logical parameter. When it is TRUE, the function save the plot in the chosen path and format. By default is TRUE.}

\item{plot_format}{File format for the output plots (accepts "png", "jpg", "pdf", "tiff"). By default is "png"}

\item{plot_path}{Path to save output plots.}

\item{verbose}{print information about the processes of the function. By default is FALSE}
}
\value{
Plot with chromosomes clustered
}
\description{
The function clusters chromosomes based on the copy number (CN) and returns a graph where it is possible to observe the different groups and two data frames (report and plot_table). See the vignette for the data frame descriptions.
}
\examples{
data(TCGA_BRCA_CN_segments)
Cluster <- PlotChrCluster(segs=TCGA_BRCA_CN_segments, 
                         clust_method= "ward.D2", 
                         plot_output=FALSE)
}
